%{
#include <stdio.h> /* for printf() */
#include <stdlib.h>
#include "example.h"
%}

%union {
	Expr *expr;
	LogExpr *log_expr;
	Stmt *stmt;
	ListStmt *list_stmt;

	int int_val;
	char ident_name[256];
}; /* тип yylval */

/* Precedence */
%left '+' '-'
%left '*' '/'
%right POW
%nonassoc UMINUS

%token <int_val> NUMERAL
%token <ident_name> IDENT
%token PRINT IF THEN ELSE ENDIF EQ

%type <expr> expr
%type <log_expr> log_expr
%type <list_stmt> list_stmt
%type <stmt> stmt

%%

program
	: list_stmt { Program = $1; }
	;

list_stmt
	: list_stmt stmt separator { $1->add($2); $$=$1; }
	| /* epsilon */ { $$ = new ListStmt; }
	;

separator
	: '\n'
	| separator '\n'
	;

stmt
	: PRINT expr { $$ = new StmtPrint($2); } 
	| IDENT '=' expr { $$ = new StmtAssign($1, $3); }
	| IF log_expr THEN separator list_stmt ELSE separator list_stmt ENDIF { $$ = new StmtIf($2, $5, $8); }
	| IF log_expr THEN separator list_stmt ENDIF { $$ = new StmtIf($2, $5, NULL); }
	;

log_expr
	: expr '<' expr { $$ = new LogExpr('<', $1, $3); }
	| expr '>' expr { $$ = new LogExpr('>', $1, $3); }
	| expr EQ expr	{ $$ = new LogExpr('=', $1, $3); }
	;
expr
	: IDENT  { $$ = new ExprVariable($1); }
	| NUMERAL 		{ $$ = new ExprNumeral($1); }
	| '-' expr %prec UMINUS	{ $$ = new ExprArith(UMINUS, $2, NULL); }
	| expr '+' expr		{ $$ = new ExprArith('+', $1, $3); }
	| expr '-' expr		{ $$ = new ExprArith('-', $1, $3); }
	| expr '*' expr		{ $$ = new ExprArith('*', $1, $3); }
	| expr '/' expr		{ $$ = new ExprArith('/', $1, $3); }	
	| expr POW expr		{ $$ = new ExprArith(POW, $1, $3); }
	| '(' expr ')'		{ $$ = $2; }
	;

%%
