#include <iostream>
#include <stdlib.h>
#include "example.h"

int ExprArith::eval()
{
	int v1, v2;
	v1 = e1->eval(); if(e2) { v2 = e2->eval(); }
	switch(op) {
		case '+': return v1 + v2;
		case '-': return v1 - v2;
		case '*': return v1 * v2;
		case '/': 
			  if (v2 == 0) {
				  cerr << "Runtime error: division by zero\n";
				  exit(1);
			  }
			  return v1 / v2;
		case UMINUS: return -v1;
		case POW: return pow_int(v1,v2);
	}

	return 0;
}

int ExprVariable::eval()
{
	struct id_tab *tab; 
	if ((tab = get_id(name))) {
		return tab->value;
	} else {
		cerr << "Runtime error: " << name << " not defined\n";
		exit(1);
	}
}

bool LogExpr::eval()
{
	int v1, v2;
	v1 = e1->eval(); v2 = e2->eval();
	switch(op) {
		case '<': return v1 < v2;
		case '>': return v1 > v2;
		case '=': return v1 == v2;
	}

	return false;
}

void ListStmt::run()
{
	vector<Stmt*>::iterator i;
	for (i = list.begin(); i < list.end(); i++)
		(*i)->run();
}

void StmtPrint::run()
{
	cout << e->eval() << '\n';
}

void StmtIf::run()
{
	bool v = e->eval();

	if (v) 
		L1->run();
	else if (L2)
		L2->run();
}

void StmtAssign::run()
{
			int value = e->eval();
			struct id_tab *tab;
			if ((tab = get_id(name))) {
				tab->value = value;
			} else {
				add_id(name, value);
			}
}


ListStmt *Program;
